﻿using Newtonsoft.Json;
using System;

namespace VPNClient
{
    class ConfigLoader
    {
        /* Singleton instance */
        private static readonly ConfigLoader m_instance = new ConfigLoader("client.cfg");

        private string m_filename;
        private ClientConfig m_config;

        private ConfigLoader(string filename)
        {
            this.m_filename = filename;

            m_config = this.LoadConfig();
        }

        /****************************************
        * Purpose: Singleton for ClientLoader
        * Input: N/A
        * Output: N/A
        * Return: ClientLoader instance
        * Author: Jemin Su
        * Date: 2017-04-06
        ****************************************/
        public static ConfigLoader GetInstance()
        {
            return m_instance;
        }

        /****************************************
        * Purpose: Load the VPN Client config
        * Input: N/A
        * Output: N/A
        * Return: ClientConfig
        * Author: Jemin Su
        * Date: 2016-09-01
        ****************************************/
        private ClientConfig LoadConfig()
        {
            ClientConfig config = null;

            try
            {
                // Read the config file
                System.IO.StreamReader sr = new System.IO.StreamReader(m_filename);
                config = JsonConvert.DeserializeObject<ClientConfig>(sr.ReadToEnd());
                sr.Close();

            }
            catch (Exception ex)
            {
                config = new ClientConfig();

                // Default setting
                config.type = ActivationType.MANUAL;
                config.save_pwd = false;
                config.save_uid = false;
                config.server = "";
            }

            return config;
        }

        /****************************************
        * Purpose: Get the VPN Client config
        * Input: N/A
        * Output: N/A
        * Return: ClientConfig
        * Author: Jemin Su
        * Date: 2017-04-06
        ****************************************/
        public ClientConfig GetConfig()
        {
            return this.m_config;
        }

        /****************************************
        * Purpose: Save the VPN Client config
        * Input: config - the ClientConfig
        *        filename - the config file path
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-09-01
        ****************************************/
        public void SaveConfig(ClientConfig config)
        {
            try
            {
                string content = JsonConvert.SerializeObject(config);

                System.IO.StreamWriter sw = new System.IO.StreamWriter(m_filename);
                sw.Write(content);
                sw.Flush();
                sw.Close();
            }
            catch (Exception ex)
            {

            }
            finally
            {
                this.m_config = config;
            }
        }
    }
}
