﻿using BrightIdeasSoftware;
using Newtonsoft.Json;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;

namespace VPNClient
{
    public partial class TabDevicePool : RoundTab
    {
        private VPNCloudService m_service = null;
        private static TabDevicePool m_instance = new TabDevicePool();

        public TabDevicePool()
        {
            InitializeComponent();

            this.m_service = VPNCloudService.GetInstance();

            /* Setup the tree view expand */
            this.treeListViewDevice.CanExpandGetter = delegate (object x) {
                DeviceTreeNode node = (DeviceTreeNode)x;
                if (node.gateway != null)
                {
                    if(node.gateway.Devices.Count > 0)
                    {
                        return true;
                    }
                }
                return false;
            };

            /* Setup the tree view clild fetch */
            this.treeListViewDevice.ChildrenGetter = delegate (object x) {
                try
                {
                    DeviceTreeNode node = (DeviceTreeNode)x;
                    return node.devices;
                }
                catch (UnauthorizedAccessException ex)
                {
                    this.BeginInvoke((MethodInvoker)delegate () {
                        this.treeListViewDevice.Collapse(x);
                        MessageBox.Show(this, ex.Message, "treeListViewDevice", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    });
                    return new ArrayList();
                }
            };
            
            /* Set the row height to 26 pixels */
            this.treeListViewDevice.RowHeight = 26;

            this.olvName.AspectGetter = delegate (object x)
            {
                DeviceTreeNode node = (DeviceTreeNode)x;
                if (x != null)
                {
                    if(node.gateway == null)
                    {
                        /* Device */
                    }
                    else
                    {
                        /* Gateway */
                    }
                    return node.name;
                }
                    
                return "-";
            };

            this.olvType.ImageGetter = delegate (object x)
            {
                DeviceTreeNode node = (DeviceTreeNode)x;
                int index = node.status * 3 + node.type;
                if(index >= 0 && index < this.imageListType.Images.Count)
                    return this.imageListType.Images[index];
                else
                    return null;
            };

            this.olvAction.ImageGetter = delegate (object x)
            {
                DeviceTreeNode node = (DeviceTreeNode)x;
                int index = ConnectionStatus.UNCONNECTED;
                if (node.connected)
                    return this.imageListAction.Images[ConnectionStatus.CONNECTED];
                return this.imageListAction.Images[index];
            };

            /*this.olvAction.AspectGetter = delegate (object x) {
                DeviceTreeNode node = (DeviceTreeNode)x;
                if (x != null)
                    return node.action;
                return "-";
            };
            FlagRenderer attributesRenderer = new FlagRenderer();
            attributesRenderer.ImageList = imageListSmall;
            attributesRenderer.Add(1, "connect");
            attributesRenderer.Add(0, "disconnect");
            this.olvAction.Renderer = attributesRenderer;*/

            /* Remove UI
            this.olvConnected.AspectGetter = delegate (object x)
            {
                DeviceTreeNode node = (DeviceTreeNode)x;
                if (node != null)
                {
                    if(node.connected)
                        return ConnectionStatus.STRING[ConnectionStatus.CONNECTED];
                }

                return ConnectionStatus.STRING[ConnectionStatus.UNCONNECTED];
            };*/

            this.olvVIP.AspectGetter = delegate (object x)
            {
                DeviceTreeNode node = (DeviceTreeNode)x;
                if (x != null)
                    return node.vip;
                return "-";
            };

            this.olvIP.AspectGetter = delegate (object x)
            {
                DeviceTreeNode node = (DeviceTreeNode)x;
                if (x != null)
                    return node.ip;
                return "-";
            };

            SetupWithButton();
        }

        public static TabDevicePool GetInstance()
        {
            return m_instance;
        }

        private void SetupWithButton()
        {
            // Tell the columns that it is going to show buttons.
            // The label that goes into the button is the Aspect that would have been
            // displayed in the cell.
            this.olvAction.IsButton = false;

            // How will the button be sized? That can either be:
            //   - FixedBounds. Each button is ButtonSize in size
            //   - CellBounds. Each button is as wide as the cell, inset by CellPadding
            //   - TextBounds. Each button resizes to match the width of the text plus ButtonPadding
            //this.olvAction.ButtonSizing = OLVColumn.ButtonSizingMode.FixedBounds;
            //this.olvAction.ButtonSize = new Size(40, 18);
            this.olvAction.ButtonSizing = OLVColumn.ButtonSizingMode.CellBounds;

            // Make the buttons clickable even if the row itself is disabled
            this.olvAction.EnableButtonWhenItemIsDisabled = true;
            //this.olvAction.AspectName = "NextAction";
            //this.olvAction.ImageAspectName = "connect";
            this.olvAction.TextAlign = HorizontalAlignment.Center;

            this.treeListViewDevice.CellClick += delegate (object sender, CellClickEventArgs e)
            {
                /* Connect has been pressed */
                if(e.ColumnIndex == 2)
                {
                    DeviceTreeNode node = (DeviceTreeNode)e.Model;
                    node.connected = !node.connected;

                    if(node.connected)
                        m_service.ConnectDevice(node);
                    else
                        m_service.DisconnectDevice(node);

                    /* Gateway will connect / disconnect all devices under it */
                    if (node.devices != null)
                    {
                        for (int i = 0; i < node.devices.Count; i++)
                        {
                            DeviceTreeNode device = (DeviceTreeNode)node.devices[i];
                            device.connected = node.connected;

                            if (device.connected)
                                m_service.ConnectDevice(device);
                            else
                                m_service.DisconnectDevice(device);
                        }
                    }

                    /* Send the connect/disconnect request to remote server */

                }
            };

            // Listen for button clicks -- which for the purpose of the demo will cycle the state of the service task
            this.treeListViewDevice.ButtonClick += delegate (object sender, CellClickEventArgs e) {
                /*Coordinator.ToolStripStatus1 = String.Format("Button clicked: ({0}, {1}, {2})", e.RowIndex, e.SubItem, e.Model);

                // We only have one column with a button, but if there was more than one, you would have to check ColumnIndex to see which button was clicked
                ServiceTask task = (ServiceTask)e.Model;
                task.AdvanceToNextState();

                // Just to show off disabled rows, make tasks that are frozen be disabled.
                if (task.Status == ServiceTask.TaskStatus.Frozen)
                    this.treeListViewDevice.DisableObject(e.Model);
                else
                    this.treeListViewDevice.EnableObject(e.Model);*/

                this.treeListViewDevice.RefreshObject(e.Model);
            };

            this.treeListViewDevice.UseCellFormatEvents = true;
            this.treeListViewDevice.FormatCell += delegate (object sender, FormatCellEventArgs e)
            {
                if(e.ColumnIndex == 0)
                {
                    /* Name */
                    DeviceTreeNode node = (DeviceTreeNode)e.Model;
                    if(node.gateway == null)
                    {
                        /* Device */
                        e.SubItem.Font = new Font("Verdana", 9, FontStyle.Regular);
                    }
                    else
                    {
                        /* Gateway */
                        e.SubItem.Font = new Font("Verdana", 9, FontStyle.Bold);
                        
                    }
                }
                else if(e.ColumnIndex == 3)
                {
                    /* Status cell */
                    DeviceTreeNode node = (DeviceTreeNode)e.Model;
                    if(node.connected)
                    {
                        e.SubItem.ForeColor = Color.Green;
                    }
                    else if(!node.connected)
                    {
                        e.SubItem.ForeColor = Color.Red;
                    }
                }
            };
        }

        public string CalculateVIP(int type, int group, int gateway, int device)
        {
            if(type == ClientType.GATEWAY)
            {
                /* VPN Gateway */
                string prefix = (group >> 8) + "." + (group % 256);
                int ip3 = 0, ip4 = 0;
                if (gateway > 0)
                {
                    ip3 = ((gateway - 1) / 8) + 1;
                    ip4 = (((gateway - 1) % 8) << 5) + 1 + device;
                }
                return prefix + "." + ip3 + "." + ip4;
            }
            else
            {
                /* VPN Client */
                if (gateway > 0 && gateway <= 250)
                {
                    var ip4 = gateway + 4;
                    return ((group >> 8) + "." + (group % 256) + ".0." + ip4);
                }
                else
                {
                    return "10.0.0.254";
                }
            }

        }

        /****************************************
        * Purpose: Reload device pool
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-23
        ****************************************/
        private void ReloadDevicePool()
        {
            /* Get the latest device status */
            DeviceGroup deviceGroup = this.m_service.GetDeviceGroup();

            if (deviceGroup == null)
            {
                this.treeListViewDevice.Roots = new ArrayList();
                return;
            }
            if (deviceGroup.Clients == null)
            {
                this.treeListViewDevice.Roots = new ArrayList();
                return;
            }

            List<DeviceTreeNode> roots = new List<DeviceTreeNode>();
            for (int i = 0; i < deviceGroup.Clients.Count; i++)
            {
                ClientGateway gateway = deviceGroup.Clients[i];
                DeviceTreeNode node = new DeviceTreeNode();
                node.name = gateway.name;
                node.type = gateway.type;
                node.status = gateway.status;
                if (gateway.status == OnlineStatus.UNKNOWN)
                    node.status = OnlineStatus.OFFLINE;
                node.vip = this.CalculateVIP(gateway.type, deviceGroup.network, gateway.network, 0);
                if(gateway.auto_mapping == false)
                {
                    if(gateway.Devices.Count == 0)
                    {
                        /* Site-to-Site */
                        node.ip = gateway.local_ip;
                    }
                    else
                    {
                        node.ip = "-";
                    }
                }
                else
                {
                    node.ip = "-";
                }
                node.gateway = gateway;
                node.connected = false;
                roots.Add(node);

                if (gateway.Devices != null)
                {
                    List<DeviceTreeNode> devices = new List<DeviceTreeNode>();
                    for (int j = 0; j < gateway.Devices.Count; j++)
                    {
                        DeviceTreeNode device = new DeviceTreeNode();
                        device.type = ClientType.DEVICE;
                        device.connected = false;
                        device.name = node.gateway.Devices[j].name;
                        if (gateway.status == OnlineStatus.ONLINE)
                            device.status = gateway.Devices[j].status;
                        else
                            device.status = OnlineStatus.OFFLINE;

                        if(gateway.Devices[j].type == 1) /* L2 Device */
                        {
                            device.vip = "-";
                        }
                        else /* IP Device */
                        {
                            if (gateway.auto_mapping)
                                device.vip = this.CalculateVIP(gateway.type, deviceGroup.network, gateway.network, gateway.Devices[j].network);
                            else
                                device.vip = "-";
                        }

                        if (gateway.Devices[j].type == 1) /* L2 Device */
                            device.ip = gateway.Devices[j].device_mac;
                        else
                            device.ip = gateway.Devices[j].device_ip;
                            
                        devices.Add(device);
                    }
                    node.devices = devices;
                }
            }

            this.treeListViewDevice.Roots = roots;

            /* Load the previous device tree */
            List<DeviceTreeNode> oldRoot = this.m_service.GetDevicePoolList();


            if(oldRoot != null)
            {
                /* Scan gateway list */
                for (int i = 0; i < roots.Count; i++)
                {
                    DeviceTreeNode gateway = roots[i];
                    for (int j = 0; j < oldRoot.Count; j++)
                    {
                        DeviceTreeNode oldgateway = oldRoot[j];
                        if (gateway.gateway.id == oldgateway.gateway.id)
                        {
                            /* Keep the connection status */
                            gateway.connected = oldgateway.connected;

                            /* Scan device list */
                            for (int x = 0; x < gateway.devices.Count; x++)
                            {
                                DeviceTreeNode device = gateway.devices[x];
                                for (int y = 0; y < oldgateway.devices.Count; y++)
                                {
                                    DeviceTreeNode oldDevice = oldgateway.devices[y];
                                    if (device.name.Equals(oldDevice.name) == true)
                                    {
                                        /* Keep the device connection status */
                                        device.connected = oldDevice.connected;
                                    }
                                    break;
                                }
                            }

                            break;
                        }
                    }
                }
            }
            
            /* Store the device pool list */
            this.m_service.SetDevicePoolList(roots);
        }

        /****************************************
        * Purpose: Reload device pool with snapshot
        * Input: snapshot list
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2016-08-23
        ****************************************/
        private void ReloadDevicePoolWithSnapshot(List<DeviceTreeNode> list)
        {
            /* Get the latest device status, and update their status to the snapshot devices */
            DeviceGroup deviceGroup = this.m_service.GetDeviceGroup();
            for (int i = 0; i < list.Count; i++)
            {
                DeviceTreeNode gatewayNode = (DeviceTreeNode)list[i];
                int j = 0;
                for (; j < deviceGroup.Clients.Count; j++)
                {
                    ClientGateway gateway = deviceGroup.Clients[j];
                    if (gatewayNode.gateway.id == gateway.id)
                    {
                        /* Update the gateway status */
                        gatewayNode.gateway = gateway;
                        gatewayNode.status = gateway.status;
                        if (gateway.status == OnlineStatus.UNKNOWN)
                            gatewayNode.status = OnlineStatus.OFFLINE;

                        /* For updating the device status */
                        for (int x = 0; x < gatewayNode.devices.Count; x++)
                        {
                            DeviceTreeNode deviceNode = (DeviceTreeNode)gatewayNode.devices[x];
                            int y = 0;
                            for (; y < gateway.Devices.Count; y++)
                            {
                                RemoteDevice device = gateway.Devices[y];
                                if (deviceNode.name.Equals(device.name) == true)
                                {
                                    /* Update the device status */
                                    if (gateway.status == OnlineStatus.ONLINE)
                                        deviceNode.status = device.status;
                                    else
                                        deviceNode.status = OnlineStatus.OFFLINE;

                                    break;
                                }
                            }
                            /* The snapshot device has no status information */
                            if (y >= gateway.Devices.Count)
                            {
                                deviceNode.status = OnlineStatus.OFFLINE;
                            }
                        }

                        break;
                    }
                }

                /* The snapshot gateway has no status information */
                if(j >= deviceGroup.Clients.Count)
                {
                    gatewayNode.status = OnlineStatus.OFFLINE;
                    for(int k = 0; k < gatewayNode.devices.Count; k++)
                    {
                        gatewayNode.devices[k].status = OnlineStatus.OFFLINE;
                    }
                }
            }

            this.m_service.SetDevicePoolList(list);
            this.treeListViewDevice.Roots = list;
        }

        public void SetupDevicePool ()
        {
            if (this.m_service.GetDevicePoolList() == null)
            {
                List<DeviceTreeNode> list = null;
                /* Try to load the snapshot if exists */
                try
                {
                    String filename = this.m_service.GetSnapshotFilename();
                    if (filename != null)
                    {
                        System.IO.StreamReader sr = new System.IO.StreamReader(filename);
                        list = JsonConvert.DeserializeObject<List<DeviceTreeNode>>(sr.ReadToEnd());
                        sr.Close();
                    }
                }
                catch (Exception ex)
                {
                }

                if (list != null)
                {
                    this.ReloadDevicePoolWithSnapshot(list);
                }
                else
                {
                    this.ReloadDevicePool();
                }
            }
        }

        /****************************************
        * Purpose: Handle the event on the tab visible changed
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-16
        ****************************************/
        private void TabDevicePool_VisibleChanged(object sender, EventArgs e)
        {
            TabDevicePool tab = (TabDevicePool)sender;

            if (tab.Visible)
            {
                /* Device Pool TAB switch to visible */
                if(this.m_service.GetDevicePoolList() == null)
                {
                    List<DeviceTreeNode> list = null;
                    /* Try to load the snapshot if exists */
                    try
                    {
                        String filename = this.m_service.GetSnapshotFilename();
                        if(filename != null)
                        {
                            System.IO.StreamReader sr = new System.IO.StreamReader(filename);
                            list = JsonConvert.DeserializeObject<List<DeviceTreeNode>>(sr.ReadToEnd());
                            sr.Close();
                        }
                    }
                    catch (Exception ex)
                    {
                    }
                    
                    if(list != null)
                    {
                        this.ReloadDevicePoolWithSnapshot(list);
                    }
                    else
                    {
                        this.ReloadDevicePool();
                    }
                }
                else
                {
                    /* Update the tree view again */
                    this.treeListViewDevice.Roots = this.m_service.GetDevicePoolList();
                }
            }
        }

        /****************************************
        * Purpose: Handle the refresh button
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-02-16
        ****************************************/
        private void btnRefresh_Click(object sender, EventArgs e)
        {
            /* Refresh only if the VPN tunnel is established */
            if(this.m_service.IsSignedIn() == true)
            {
                ReloadDevicePool();
            }
        }

        /****************************************
        * Purpose: Handle the snapshot button
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-03-31
        ****************************************/
        private void btnSnapshot_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Device Pool status has been saved as default", "Snapshot", MessageBoxButtons.OK, MessageBoxIcon.Information);

            List<DeviceTreeNode> list = this.m_service.GetDevicePoolList();
            if(list != null)
            {
                string filename = this.m_service.GetSnapshotFilename();
                if(filename != null)
                {
                    System.IO.StreamWriter sw = new System.IO.StreamWriter(filename);

                    /* Save the device list as JSON format */
                    String content = JsonConvert.SerializeObject(list);
                    sw.Write(content);

                    sw.Flush();
                    sw.Close();
                }
                else
                {
                    MessageBox.Show("Save snapshot failure", "Snapshot", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
        }

        /****************************************
        * Purpose: Handle the device pool save button
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-03-31
        ****************************************/
        private void btnSave_Click(object sender, EventArgs e)
        {
            List<DeviceTreeNode> list = this.m_service.GetDevicePoolList();

            if(list == null)
            {
                MessageBox.Show("Device Pool is empty", "Save device pool", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            else
            {
                if (this.saveFileDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    try
                    {
                        System.IO.StreamWriter sw = new System.IO.StreamWriter(this.saveFileDlg.FileName);

                        /* Save the device list as JSON format */
                        String content = JsonConvert.SerializeObject(list);
                        sw.Write(content);

                        sw.Flush();
                        sw.Close();
                    }
                    catch(Exception ex)
                    {

                    }
                }
            }
        }

        /****************************************
        * Purpose: Handle the device pool load button
        * Input: N/A
        * Output: N/A
        * Return: N/A
        * Author: Jemin Su
        * Date: 2017-03-31
        ****************************************/
        private void btnLoad_Click(object sender, EventArgs e)
        {
            if (this.openFileDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                List<DeviceTreeNode> list = null;
                try
                {
                    System.IO.StreamReader sr = new System.IO.StreamReader(this.openFileDlg.FileName);
                    list = JsonConvert.DeserializeObject<List<DeviceTreeNode>>(sr.ReadToEnd());

                    sr.Close();
                }
                catch(Exception ex)
                {

                }

                /* Replace the device list */
                if(list != null)
                {
                    this.m_service.SetDevicePoolList(list);
                    this.treeListViewDevice.Roots = list;
                }
            }
        }
    }
}
