﻿using System.Collections;
using System.Collections.Generic;

namespace VPNClient
{
    static class ActivationType
    {
        public const int MANUAL = 1;
        public const int KEY = 2;
    }

    static class ClientType
    {
        public const int DEVICE = 0;
        public const int GATEWAY = 1;
        public const int CLIENT = 2;

        public static string[] STRING = {
            "Device",
            "Gateway",
            "Client"
        };
    }

    static class OnlineStatus
    {
        public const int UNKNOWN = 0;
        public const int ONLINE = 1;
        public const int OFFLINE = 2;

        public static string[] STRING = {
            "Unknown",
            "Online",
            "Offline"
        };
    }

    static class ConnectionStatus
    {
        public const int UNCONNECTED = 0;
        public const int CONNECTED = 1;

        public static string[] STRING = {
            "Closed",
            "Open"
        };
    }

    class ClientConfig
    {
        public int type { get; set; }
        public string keyfile { get; set; }
        public string server { get; set; }
        public string group { get; set; }
        public string uid { get; set; }
        public string pwd { get; set; }
        public string code { get; set; }
        public bool save_uid { get; set; }
        public bool save_pwd { get; set; }
        public string key { get; set; }

        public string iface { get; set; }

        public bool start_as_service { get; set; }
        public bool client_to_client { get; set; }

        public List<ClientGateway> Clients { get; set; }
    }

    class ActivationKey
    {
        public string server { get; set; }
        public string group { get; set; }
        public string uid { get; set; }
        public string name { get; set; }
        public string crt { get; set; }
        public string key { get; set; }
        public int id { get; set; }
        public string cn { get; set; }
    }

    class ActivationResponse
    {
        public bool error { get; set; }
        public string message { get; set; }
        public string key { get; set; }
        public string ovpn { get; set; }
        public string ip { get; set; }
        public string network { get; set; }
    }

    class RemoteDevice
    {
        public int id { get; set; }
        public string name { get; set; }
        public string device_ip { get; set; }
        public string device_mac { get; set; }
        public int status { get; set; }
        public int type { get; set; }
        public int network { get; set; }
        public bool pool { get; set; }
        public bool connected { get; set; }
    }

    class ClientGateway
    {
        public int id { get; set; }
        public string name { get; set;  }
        public int type { get; set; }
        public int network { get; set; }
        public int status { get; set; }
        public int valid { get;  set; }
        public bool auto_mapping { get; set; }
        public string local_ip { get; set; }
        public string local_mask { get; set; }
        public List<RemoteDevice> Devices { get; set; }
        public bool pool { get; set; }
        public bool connected { get; set; }
    }

    class DeviceGroup
    {
        public bool error { get; set; }
        public string message { get; set; }
        public string name { get; set; }
        public int network { get; set; }
        public List<ClientGateway> Clients { get; set; }
    }

    class DeviceTreeNode
    {
        public string name { get; set; }
        public bool connected { get; set; }
        public int status { get; set; }
        public string vip { get; set; }
        public string ip { get; set; }
        public int type { get; set; }
        public ClientGateway gateway { get; set; }
        public List<DeviceTreeNode> devices { get; set; }
    }
}
